/*
-------------------------------------------------------------------------------
    This file is a part of the Vst Preset Generator
    Copyright (C) 2007  Francois Mazen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-------------------------------------------------------------------------------
  Name: maindialog.cpp
  Author: Francois Mazen
  Date: 28/05/07 16:25
  Description: the MainDialog class, the core...
-------------------------------------------------------------------------------
*/

#include <QtGui>

#include "maindialog.h"
#include "vstpresetgenerator.h"
#include "vpgxml.h"
#include "vpgtools.h"

//-----------------------------------------------------------------------------
//MainDialog Class
//-----------------------------------------------------------------------------

//constructor
MainDialog::MainDialog(QWidget *parent) : QMainWindow(parent)
{
     //Qt objects------------------------------------------
        //buttons:
     //btnQuit = new QPushButton(tr("&Quit"));
     //btnOpen = new QPushButton(tr("&Import informations\nfrom a preset file"));
     //btnLoadVst = new QPushButton(tr("Import informations\nfrom a plugin &VST"));
     //btnOpenList = new QPushButton(tr("&Open a Vst list"));
     //btnSaveList = new QPushButton(tr("&Save the Vst list"));
     //btnNewList = new QPushButton(tr("&New Vst List"));
     btnGen = new QPushButton(QIcon(":/images/generate.png"),tr("&Generate a preset file !"));
     btnGen->setIconSize (QSize(32,32));
        //label:
     headerDescription = new QLabel(tr("No header loaded"));
     
        //containers:
     QWidget *mainContainer = new QWidget(this);
     QWidget *bottomContainer = new QWidget(mainContainer);
     QWidget *topContainer = new QWidget(mainContainer);
     //QWidget *listContainer = new QWidget(topContainer);
     //QWidget *headerContainer = new QWidget(topContainer);
     
        //groupBox:
     //QGroupBox *gbFileType = new QGroupBox(tr("File Type"),topContainer);
        //layout:     
     QHBoxLayout *mainLayout = new QHBoxLayout;
     //QGridLayout *bottomLayout = new QGridLayout;//2 rows, 3 columns
     QVBoxLayout *bottomLayout = new QVBoxLayout;
     QVBoxLayout *topLayout = new QVBoxLayout;
     //QGridLayout *listLayout = new QGridLayout;//2 rows, 2 columns
     //QGridLayout *headerLayout = new QGridLayout;//2 rows, 2 columns
       
       //Scroll Area
     //scrollArea = new QScrollArea(this);
     //paramContainer = new QFrame();
     //QVBoxLayout *paramLayout = new QVBoxLayout(paramContainer);
     //paramContainer->setLayout(paramLayout);
       //Progress Bar
     progressBar = new QProgressBar(bottomContainer);
       //List
     vstList = new QListView();
       //Image
     vpgLogo = new const QPixmap(VPG_LOGO);
     QLabel *lblLogo = new QLabel;
     lblLogo->setPixmap(*vpgLogo);
     
     //scrollArea->setWidget(paramContainer);
     //scrollArea->setWidgetResizable(true);
     
       //Random Manager
     //vpgRandom = new VpgRandom();
       //Vst host
     vpgVstHost = new VpgVstHost();
     
       //file dialog
     fileTypeDialog = new FileTypeDialog(this);
       //table for parameters diplay
     paramTable = new QTableView(this);
     paramManager = new ParamManager();
     paramValueDelegate = new ParamValueDelegate(this);
     randomModeDelegate = new RandomModeDelegate(this);
     paramTable->setModel(paramManager);
     //paramTable->setItemDelegate(paramValueDelegate);
     //paramTable->setItemDelegate(randomModeDelegate);
	 paramTable->setItemDelegateForColumn(0,paramValueDelegate);
	 paramTable->setItemDelegateForColumn(1,randomModeDelegate);
     paramTable->setItemDelegateForColumn(2,paramValueDelegate);
     paramTable->setItemDelegateForColumn(3,paramValueDelegate);
     
     
     
    
     
     //Fill layout------------------------------------------
       //list layout
     //listLayout->addWidget(btnOpenList,0,0,1,1);
     //listLayout->addWidget(btnSaveList,0,1,1,1);
     //listLayout->addWidget(btnNewList,0,2,1,1);
     //listLayout->addWidget(vstList,1,0,1,3);
     //listContainer->setLayout(listLayout);
       //header layout
     //headerLayout->addWidget(btnOpen, 0,0,1,1);
     //headerLayout->addWidget(btnLoadVst,0,1,1,1);
     //headerLayout->addWidget(headerDescription,1,1,1,1);
     //headerLayout->addWidget(lblLogo,1,0,1,1);
     //headerLayout->setAlignment(Qt::AlignHCenter);
     //headerContainer->setLayout(headerLayout);
       //groupbox
     //fileTypeLayout->addWidget(rbtnProgram,0,0,1,1);
     //fileTypeLayout->addWidget(rbtnBank,1,0,1,1);
     //fileTypeLayout->addWidget(lblNbProg,2,0);
     //fileTypeLayout->addWidget(spbxBank,2,1);
     //gbFileType->setLayout(fileTypeLayout);
       //topLayout
     //topLayout->addWidget(headerContainer);
     //topLayout->addWidget(listContainer);
     topLayout->addWidget(vstList);
     //topLayout->addWidget(gbFileType);
     topLayout->addWidget(headerDescription);
	 topLayout->addWidget(btnGen);
     //topLayout->setSizeConstraint(QLayout::SetFixedSize);
     topContainer->setLayout(topLayout);
       //bottom layout   
     //bottomLayout->addWidget(scrollArea,0,0,1,3);
     //bottomLayout->addWidget(scrollArea);
     bottomLayout->addWidget(paramTable);
     //bottomLayout->addWidget(btnGen,1,0);
     //bottomLayout->addWidget(progressBar,1,1);
     bottomLayout->addWidget(progressBar);
     //bottomLayout->addWidget(btnQuit,1,2);
     bottomContainer->setLayout(bottomLayout);
       //global layout
     //mainLayout->addWidget(topContainer);
     mainLayout->addWidget(bottomContainer);
     mainContainer->setLayout(mainLayout);
     setCentralWidget(mainContainer);
     
     //dock ----------------------------------------------
     QDockWidget *dockList = new QDockWidget(tr("VST List"), this);
     dockList->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
     dockList->setFeatures(QDockWidget::DockWidgetMovable);
     dockList->setWidget(topContainer);
     addDockWidget(Qt::LeftDockWidgetArea, dockList);
     
     
     //init values----------------------------------------
     //gbFileType->setEnabled(false);
     //scrollArea->setEnabled(false);
     btnGen->setEnabled(false);
     lastDir = "";
     
     //vst list
     vpgXml = new VpgXml(this);
     //vpgXml->ReadFile(VST_LIST_FILENAME);
     vstList->setModel(vpgXml);
     
     QString title(tr("Vst Preset Generator V"));
     title += VSTPRESETGENERATORVERSION;
     setWindowTitle(title);
     fxHeader = new FxHeader();
     FxHeaderStruct t_fxheaderstruct;
     t_fxheaderstruct.name = "";
     t_fxheaderstruct.fxID = "";
     t_fxheaderstruct.fxVersion = 0;
     t_fxheaderstruct.nParams = 0;
     t_fxheaderstruct.useChunk = false;
     ShowLabelHeader(&t_fxheaderstruct);
     
     //Menus-------------------------------------------------
       //Import
     //QAction *importPreset = new QAction(tr("Import informations from a &preset file"),this);
     QAction *importVst = new QAction(QIcon(":/images/import.png"),tr("Import informations about &VST Plugin"),this);
     QMenu *importMenu = menuBar()->addMenu(tr("&Import"));
     //importMenu->addAction(importPreset);
     importMenu->addAction(importVst);
       //Vst List
     QAction *openVstListAction = new QAction(QIcon(":/images/open.png"),tr("&Open VST List..."), this);
     QAction *saveVstListAction = new QAction(QIcon(":/images/save.png"),tr("&Save VST List..."), this);
     QAction *newVstListAction = new QAction(QIcon(":/images/new.png"),tr("&New VST List"), this);
     QMenu *vstListMenu = menuBar()->addMenu(tr("&VST List"));
     vstListMenu->addAction(newVstListAction);
     vstListMenu->addAction(openVstListAction);
     vstListMenu->addAction(saveVstListAction);
       //About
     QAction *aboutAction = new QAction(tr("&About VST Preset Generator"), this);
     QAction *aboutQtAction = new QAction(tr("About &Qt"), this);
     QMenu *aboutMenu = menuBar()->addMenu(tr("&About"));
     aboutMenu->addAction(aboutAction);
     aboutMenu->addAction(aboutQtAction);
     
     //Toolbars-----------------------------------------------
     vstListToolBar = addToolBar(tr("Vst list"));
     vstListToolBar->addAction(newVstListAction);
     vstListToolBar->addAction(openVstListAction);
     vstListToolBar->addAction(saveVstListAction);
	 vstImportToolBar = addToolBar(tr("Import Vst"));
     vstImportToolBar->addAction(importVst);
     
     
     
     //Qt connexions------------------------------------------
       //buttons:
     //connect(btnQuit, SIGNAL(clicked()), this, SLOT(close()));
     //connect(btnOpen, SIGNAL(clicked()), this, SLOT(OpenPreset()));
     connect(btnGen, SIGNAL(clicked()), this, SLOT(GeneratePreset()));
     
     //connect(btnOpenList, SIGNAL(clicked()), this, SLOT(OpenVstList()));
     //connect(btnSaveList, SIGNAL(clicked()), this, SLOT(SaveVstList()));
     //connect(btnNewList, SIGNAL(clicked()), this, SLOT(NewVstList()));
     //connect(btnLoadVst, SIGNAL(clicked()),this, SLOT(LoadParamFromVst()));
       //headers:
     connect(this, SIGNAL(validHeader(FxHeaderStruct*)), 
             fxHeader, SLOT(receiveParameters(FxHeaderStruct*)));
     connect(this, SIGNAL(validHeader(FxHeaderStruct*)), 
             this, SLOT(ShowLabelHeader(FxHeaderStruct*)));
     connect(vpgXml, SIGNAL(validHeader(FxHeaderStruct*)), 
             fxHeader, SLOT(receiveParameters(FxHeaderStruct*)));
     connect(vpgXml, SIGNAL(validHeader(FxHeaderStruct*)), 
             this, SLOT(ShowLabelHeader(FxHeaderStruct*)));
     //connect(fxHeader, SIGNAL(headerIsValid(bool)), 
     //        gbFileType, SLOT(setEnabled(bool)));
     //connect(fxHeader, SIGNAL(headerIsValid(bool)), 
     //        scrollArea, SLOT(setEnabled(bool)));
     connect(fxHeader, SIGNAL(headerIsValid(bool)), 
             btnGen, SLOT(setEnabled(bool)));
     connect(fxHeader, SIGNAL(headerIsValid(bool)),
             this, SLOT(UpdateParamList()));
      //list interaction 
     connect(vstList, SIGNAL(doubleClicked ( const QModelIndex &)),
             vpgXml, SLOT(loadVst(const QModelIndex &)));
      //menu connections
     connect(aboutAction, SIGNAL(triggered()), this, SLOT(ShowCopyright()));
     connect(aboutQtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
     connect(openVstListAction, SIGNAL(triggered()), this, SLOT(OpenVstList()));
     connect(saveVstListAction, SIGNAL(triggered()), this, SLOT(SaveVstList()));
     connect(newVstListAction, SIGNAL(triggered()), this, SLOT(NewVstList()));
     //connect(importPreset, SIGNAL(triggered()), this, SLOT(OpenPreset()));
     connect(importVst, SIGNAL(triggered()), this, SLOT(ImportParamFromVstOrPreset()));
     
     
     resize(700,500);
     //for old 640*480 screens:              
     //setFixedSize(600,400);
     //setFixedSize(640,480);
     
     //Application icon
     setWindowIcon(QIcon(":/images/vpg_icon.png"));
     
     
             
     
     
}

//-----------------------------------------------------------------------------

void MainDialog::OpenPreset(QString fileName)
{
     QByteArray buffer;
     QString t_fxID;
     qint32 t_fxVersion ;
     qint32 t_nParams ;
     
     int ret;
     
    
     
     //get the name of the file
     //QString fileName = QFileDialog::getOpenFileName(this, tr("Open Preset"),
     //                                           lastDir,
     //                                           tr("Preset Program/Bank (*.fxp *.fxb)"));
     
     if(fileName == NULL)
        return;
        
     //open the file
     QFile file(fileName);
     if (!file.open(QIODevice::ReadOnly))
     {
         //error while opening
         ret = QMessageBox::critical(this, tr("Opening Preset File"),
                   tr("Could not read the file"));                        
         return;
     }
     
     QFileInfo info(file);
     lastDir = info.absolutePath();
     
     buffer = file.read(4);
     if(buffer.startsWith("CcnK") == false)   //'CcnK'
     {
           //not a preset file
           ret = QMessageBox::critical(this, tr("Opening Preset File"),
                   tr("This is not a valid Vst preset file")); 
           file.close();
           return;
     }
         
     buffer = file.read(4);//chunk size
     buffer = file.read(4);//type of file (bank or program)
     
     //'FxCk' (regular) or 'FPCh' (opaque chunk) Program
     //'FxBk' (regular) or 'FBCh' (opaque chunk) Bank
     if(buffer.startsWith("FxCk") == true)   
     {
              //it's a program or a bank
              buffer = file.read(4);//format version
              
              buffer = file.read(4);//fx unique ID
              t_fxID = buffer.data();
              
              buffer = file.read(4);//fx version
              t_fxVersion = import_long(buffer.data());
              
              buffer = file.read(4);//number of parameters
              t_nParams = import_long(buffer.data());
     }
     else if(buffer.startsWith("FPCh") == true)
     {
          //Opaque chunk data, not suported
          ret = QMessageBox::critical(this, tr("Loading Preset File"),
                   tr("Opaque Chunk method : please import informations from the VST file (.dll)")); 
          return;
     } 
     else if(buffer.startsWith("FxBk") == true)
     {
          //bank program
          
          //jump to the first program and read the parameters
          file.read(4*4 + 128 + 4*3);
          
          buffer = file.read(4);//format version
              
          buffer = file.read(4);//fx unique ID
          t_fxID = buffer.data();
              
          buffer = file.read(4);//fx version
          t_fxVersion = import_long(buffer.data());
              
          buffer = file.read(4);//number of parameters
          t_nParams = import_long(buffer.data());
          
     }
     else if(buffer.startsWith("FBCh") == true)
     {
          //Opaque chunk data, not suported
          ret = QMessageBox::critical(this, tr("Loading Preset File"),
                   tr("Opaque Chunk method : please import informations from the VST file (.dll)")); 
          return;
     }    
     else
     {
         //invalid file (no program or no bank)
         ret = QMessageBox::critical(this, tr("Loading Preset File"),
                   tr("This is not a valid Vst preset file")); 
         return;
     }
     
     //Get the name of Vst from the user:
     bool name_ok;
     QString vst_name = QInputDialog::getText(this, tr("Name of the Vst"),
                                          tr("Please enter the name of your Vst:"), QLineEdit::Normal,
                                          tr("My Vst"), &name_ok);
                                          
     FxHeaderStruct *t_fxheaderstruct = new FxHeaderStruct;
     t_fxheaderstruct->fxID = t_fxID;
     t_fxheaderstruct->fxVersion = t_fxVersion;
     t_fxheaderstruct->nParams = t_nParams;  
     t_fxheaderstruct->name = "Invalid name";
     t_fxheaderstruct->useChunk = false;
     if (name_ok && !vst_name.isEmpty())
     {
         t_fxheaderstruct->name = vst_name;   
     }
     
     //add the new Vst to our list:
     vpgXml->addVst(t_fxheaderstruct);
                   
     //send parmeters to the headers object
     emit validHeader(t_fxheaderstruct);
     
     file.close();
}

//-----------------------------------------------------------------------------

void MainDialog::GeneratePreset()
{
     QString fileName;
     QByteArray buffer;
     int ret, type;
     qint32 nbPrograms;
     bool preset_type_bank;
     
     VstInt32 buffer_int;
     char* buffer_char = (char*) malloc(5 * sizeof(char));
     
     //get the type of the file
     type = fileTypeDialog->exec();
     
     if(type == -1) //Cancel Button
        return;
     
     if(type == 0)
     {
        preset_type_bank = false;
        nbPrograms = 1;
     }
     else
     {
	  	 preset_type_bank = true;
	  	 nbPrograms = type;
     }
	  	 
     
     /*bool preset_type_bank = rbtnBank->isChecked();
	 					 	
     
     if(preset_type_bank)
     {
           nbPrograms = spbxBank->value();
     }
     else
     {
         nbPrograms = 1;
     }*/
     
     //get the name of file
     if(preset_type_bank)
     {
          fileName = QFileDialog::getSaveFileName(this,
                                              tr("Generate Preset Bank"),
                                              lastDir,
                                              tr("Preset Bank (*.fxb)"));
     }
     else
     {
         fileName = QFileDialog::getSaveFileName(this,
                                              tr("Generate Preset Program"),
                                              lastDir,
                                              tr("Preset Program (*.fxp)"));
     }
     
     
     if(fileName == NULL) //cancel button
        return;
     
     //open the file
     QFile file(fileName);
     
     if (!file.open(QIODevice::WriteOnly))
     {
         //error while opening
         ret = QMessageBox::critical(this, tr("Saving Preset File"),
                   tr("Could not create the file"));                        
         return;
     }
     
     QFileInfo info(file);
     lastDir = info.absolutePath();
     
    
     
     if(fxHeader->Data.useChunk == true)
     {
          buffer_int = vpgVstHost->getFxId();
          import_long_char(buffer_int,buffer_char);
          
          //Check the Vst loaded
          if((vpgVstHost->getFxId() == 0) ||
             (strcmp(buffer_char,qPrintable(fxHeader->Data.fxID))))
          {
		      
              ret = QMessageBox::warning(this, tr("Generating Preset File"),
                                          tr("Please load the VST file (.dll) corresponding to the specified choice"));
              //get the name of the file
     		  fileName = QFileDialog::getOpenFileName(this, tr("Load a Plugin VST"),
                                                fxHeader->Data.fxPath,
                                                tr("VST Plugin (*.dll)"));
              if(LoadVst(fileName) == false)
			      return;
		      
		      buffer_int = vpgVstHost->getFxId();
              import_long_char(buffer_int,buffer_char);
              
		      if(strcmp(buffer_char,qPrintable(fxHeader->Data.fxID)))
		      {
					   ret = QMessageBox::critical(this, tr("Generating Preset File"),
                                          tr("the VST file (.dll) don't have the same ID has the specified choice"
		                                     "\nPlease try again."));
                  return;
			  }
		  }
             
          if(preset_type_bank)
          {
			  WriteBankChunk(&file, nbPrograms);
          }
          else
          {
		   	  WriteProgramChunk(&file, nbPrograms);
          }
     }
     else //non-chunk generation
     {
	  	 if(preset_type_bank)
	  	 {
				 WriteBank(&file, nbPrograms);
         }
         else
         {
		  	 //QTime midnight(0, 0, 0);
     	  	 //vpgRandom->initSeed(midnight.secsTo(QTime::currentTime()));
          	 WriteProgram(&file,nbPrograms);
         }
     }
     
     
     
     
     //Operation success
     QString success(tr("The preset file:\n"));
     success.append(fileName);
     success.append(tr("\nhas been created. Enjoy !"));
     ret = QMessageBox::information(this, tr("Saving Preset File"), success);
     
     file.close();
     
}

//-----------------------------------------------------------------------------

void MainDialog::ShowLabelHeader(FxHeaderStruct* data)
{
     
     QString temp1;
     QString temp2;
     temp1 += tr("Vst name: ");
     temp1 += data->name;
     temp1 += tr("\nVst ID: ");
     temp1 += data->fxID;
     temp1 += tr("\nVst Version: ");
     temp2.setNum(data->fxVersion);
     temp1 += temp2;
     temp1 += tr("\nnumber of parameters: ");
     temp2.setNum(data->nParams);
     temp1 += temp2;
     temp1 += tr("\npreset type: ");
     if(data->useChunk)
     {
	       temp1 += tr("chunk");
     }
     else
     {
	  	 temp1 += tr("parameters");
     }
     
     headerDescription->setText(temp1);
     
}

//-----------------------------------------------------------------------------
void MainDialog::WriteProgram(QFile *file, qint32 no_program)
{
     QByteArray buffer;
     qint32 val, no_param;
     float valf;
     
     //cMagic
     buffer = "CcnK";
     file->write(buffer);
     
     //size of byte * number of parameters
     val = 4 * 5 + 28 + 4 * fxHeader->Data.nParams; 
     
     buffer.clear();
     buffer.resize(4);
     export_long(val,buffer.data());
     file->write(buffer);
     
     //'FxCk' (regular) or 'FPCh' (opaque chunk) Program
	 buffer = "FxCk";
     file->write(buffer);
     
	 ///< format version (currently 1)
	 val = 1;
	 buffer.clear();
     buffer.resize(4);
     export_long(val,buffer.data());
     file->write(buffer);
	 
	 ///< fx unique ID
	 buffer.clear();
	 buffer += fxHeader->Data.fxID;
	 file->write(buffer);
	 
	 ///< fx version
	 buffer.clear();
     buffer.resize(4);
     export_long(fxHeader->Data.fxVersion,buffer.data());
	 file->write(buffer);
     
     buffer.clear();
     buffer.resize(4);
     //< number of parameters
     export_long(fxHeader->Data.nParams,buffer.data());
     file->write(buffer);
	 
     ///< program name (null-terminated ASCII string) !!!![28]!!!!
     buffer.clear();
     buffer.setNum(no_program);
     buffer.prepend("VPG ");
     file->write(buffer,28);
     
     //Float parameters
        
     for(no_param = 0; no_param < fxHeader->Data.nParams; no_param ++)
     {
          buffer.clear();
          buffer.resize(4);
          /*
          min = paramList[no_param]->getMin();
          max = paramList[no_param]->getMax();
          valf = (((float) qrand()) / ((float)RAND_MAX)) * (max - min) + min;
          */
          //valf = paramList[no_param]->getRandomValue();
          valf = paramManager->getRandomValue(no_param);
          char* pt = (char*)&valf; 
          
          //inversion because little-endian notation
          buffer[3] = pt[0];
          buffer[2] = pt[1];
          buffer[1] = pt[2];
          buffer[0] = pt[3];
          
          file->write(buffer);
          
     }//params
        
     
}

//-----------------------------------------------------------------------------
void MainDialog::WriteProgramChunk(QFile *file, qint32 no_program)
{
     QByteArray buffer;
     long chunk_size;
	 void* chunk = (void*)malloc(65536); //must be change !
	 void** pchunk = &chunk;
     qint32 no_param,val;
     float valf;
     
     //first write the parameters to the plug-in
     //and get the program chunk
     //QTime midnight(0, 0, 0);
     //vpgRandom->initSeed(midnight.secsTo(QTime::currentTime()));
     for(no_param = 0; no_param < fxHeader->Data.nParams; no_param ++)
     {
          //valf = paramList[no_param]->getRandomValue();
          valf = paramManager->getRandomValue(no_param);
          vpgVstHost->setParameter(no_param, valf);
     }
     QString name;
	 name.setNum(no_program);
	 name.prepend("VPG ");
     vpgVstHost->setProgramName(qPrintable(name), no_program);
     chunk_size = vpgVstHost->getProgramChunk(pchunk);
     
        
     
     
     //cMagic
     buffer = "CcnK";
     file->write(buffer);
     
     //size 
     val = 4 * 5 + 28 + chunk_size; 
     buffer.clear();
     buffer.resize(4);
     export_long(val,buffer.data());
     file->write(buffer);
     
     //'FxCk' (regular) or 'FPCh' (opaque chunk) Program
	 buffer = "FPCh";
     file->write(buffer);
     
	 ///< format version (currently 1)
	 val = 1;
	 buffer.clear();
     buffer.resize(4);
     export_long(val,buffer.data());
     file->write(buffer);
	 
	 ///< fx unique ID
	 buffer.clear();
	 buffer += fxHeader->Data.fxID;
	 file->write(buffer);
	 
	 ///< fx version
	 buffer.clear();
     buffer.resize(4);
     export_long(fxHeader->Data.fxVersion,buffer.data());
	 file->write(buffer);
     
     buffer.clear();
     buffer.resize(4);
     //< number of parameters
     export_long(fxHeader->Data.nParams,buffer.data());
     file->write(buffer);
	 
     ///< program name (null-terminated ASCII string) !!!![28]!!!!
     buffer.clear();
     buffer.setNum(no_program);
     buffer.prepend("Vst Preset Generator ");
     file->write(buffer,28);
     
     //chunk size and chunk
     val = chunk_size; 
     buffer.clear();
     buffer.resize(4);
     export_long(val,buffer.data());
     file->write(buffer);
     file->write((char*)*pchunk, chunk_size);
    
     
}

//-----------------------------------------------------------------------------

void MainDialog::WriteBank(QFile *file, qint32 nb_programs)
{
     QByteArray buffer;
     qint32 val, no_program;
     
     //cMagic
     buffer = "CcnK";
     file->write(buffer);
     
     //size of byte * number of program
     val = 4 * 5 + 128 + nb_programs * (4 * 5 + 28 + 4 * fxHeader->Data.nParams);
     buffer.clear();
     buffer.resize(4);
     export_long(val,buffer.data());
     file->write(buffer);
     
     ///< 'FxCk' (regular) or 'FPCh' (opaque chunk)
     buffer = "FxBk";
     file->write(buffer);
     
	 ///< format version (currently 1)
	 val = 1;
	 buffer.clear();
     buffer.resize(4);
     export_long(val,buffer.data());
     file->write(buffer);
	 
	 ///< fx unique ID
	 buffer.clear();
	 buffer += fxHeader->Data.fxID;
	 file->write(buffer);
	 
	 ///< fx version
	 buffer.clear();
     buffer.resize(4);
     export_long(fxHeader->Data.fxVersion,buffer.data());
	 file->write(buffer);
     
     buffer.clear();
     buffer.resize(4);
     //number of programs
     export_long(nb_programs,buffer.data());
     file->write(buffer);
	 
	 //future space for bank preset
     buffer.clear();
     file->write(buffer, 128);
	 
	 //Write all programs
	 //QTime midnight(0, 0, 0);//dummy QTime object
     //qsrand(midnight.secsTo(QTime::currentTime()));
     //vpgRandom->initSeed(midnight.secsTo(QTime::currentTime()));
	 for(no_program = 0; no_program < nb_programs; no_program++)
	 {
         WriteProgram(file, no_program);
     }//programs
     
}

//-----------------------------------------------------------------------------
void MainDialog::WriteBankChunk(QFile *file, qint32 nb_programs)
{
 	 QByteArray buffer;
     long chunk_size;
	 void* chunk = (void*)malloc(65536); //must change !
	 void** pchunk = &chunk;
     qint32 no_param, no_program,val, max_programs;
     float valf;
     
     max_programs = nb_programs;
     if (max_programs > vpgVstHost->getNumberPrograms())
     {
	     max_programs = vpgVstHost->getNumberPrograms();
	 }
	 
     //first, write the parameters to the plug-in
     //and get the bank chunk
     //QTime midnight(0, 0, 0);
     //vpgRandom->initSeed(midnight.secsTo(QTime::currentTime()));
     for(no_program = 0; no_program < max_programs; no_program++)
     {
		   //select the program:
		   vpgVstHost->setProgram(no_program);
     	   for(no_param = 0; no_param < fxHeader->Data.nParams; no_param ++)
     	   {
 				//valf = paramList[no_param]->getRandomValue();
 				valf = paramManager->getRandomValue(no_param);
          		vpgVstHost->setParameter(no_param, valf);
           }
           QString name;
	 	   name.setNum(no_program);
	 	   name.prepend("VPG ");
     	   vpgVstHost->setProgramName(qPrintable(name), no_program);
      }
           
     
     chunk_size = vpgVstHost->getBankChunk(pchunk);
     
        
     
     
     //cMagic
     buffer = "CcnK";
     file->write(buffer);
     
     //size 
     val = 4 * 5 + 128 + chunk_size; 
     buffer.clear();
     buffer.resize(4);
     export_long(val,buffer.data());
     file->write(buffer);
     
     //'FxBk' (regular) or 'FBCh' (opaque chunk) Program
	 buffer = "FBCh";
     file->write(buffer);
     
	 ///< format version (currently 1)
	 val = 1;
	 buffer.clear();
     buffer.resize(4);
     export_long(val,buffer.data());
     file->write(buffer);
	 
	 ///< fx unique ID
	 buffer.clear();
	 buffer += fxHeader->Data.fxID;
	 file->write(buffer);
	 
	 ///< fx version
	 buffer.clear();
     buffer.resize(4);
     export_long(fxHeader->Data.fxVersion,buffer.data());
	 file->write(buffer);
     
     buffer.clear();
     buffer.resize(4);
     //< number of parameters
     export_long(fxHeader->Data.nParams,buffer.data());
     file->write(buffer);
	 
     //future space for bank preset
     buffer.clear();
     file->write(buffer, 128);
     
     //chunk size and chunk
     val = chunk_size; 
     buffer.clear();
     buffer.resize(4);
     export_long(val,buffer.data());
     file->write(buffer);
     file->write((char*)*pchunk, chunk_size);
}

//-----------------------------------------------------------------------------
void MainDialog::UpdateParamList()
{
     int i;
     //ParamWidget *temp;
     //QString name;
     char buffer_name[65];
     
     paramStruct buffer;
     buffer.name=" ";
     buffer.randomMode = uniform;
     buffer.min = 0;
     buffer.max = 1;
     buffer.mid = 0.5;
     buffer.dev = 0.01;
     buffer.val = 0;
     
     /*
     //delete the old widgets
     if(paramList.isEmpty() == false)
     {
            for(i=0;i < paramList.size(); i++)
            {
               paramContainer->layout()->removeWidget(paramList[i]);
               delete(paramList[i]);
            }
     }     
     */
     
     paramManager->clear();
     paramManager->setParamNb(fxHeader->Data.nParams);
     
     
     //paramList.clear();
     //paramList.resize(fxHeader->nParams);
     
     //avoid problems while using a no-chunk-plug-in
     if(fxHeader->Data.useChunk == false)
	      vpgVstHost->Close();
     
     progressBar->setMinimum(0);
     progressBar->setMaximum(fxHeader->Data.nParams);
     
     for(i =1; i<=fxHeader->Data.nParams; i++)
     {
           progressBar->setValue(i);
           
           //if we have loaded the plugin (and not only a preset file)
           //we get the name of each parameter
           buffer.name = tr("unknow");
           if(fxHeader->Data.useChunk == true && vpgVstHost->isVstLoaded())
           {
				 vpgVstHost->getParamName(i-1, buffer_name);
				 buffer.name = buffer_name;
           }           
           //int ret = QMessageBox::information(this, tr("Loading plugin VST"), name);
           
           paramManager->add(i,buffer);
           
           //temp = new ParamWidget(i, vpgRandom, name);
           //paramList.append(temp);
           //paramContainer->layout()->addWidget(temp);
           //paramContainer->show();
     }
     
     paramTable->resizeColumnsToContents ();
     paramTable->resizeRowsToContents();
     
     
}

//-----------------------------------------------------------------------------
void MainDialog::SaveVstList()
{
     //get the name of the file
     QString fileName = QFileDialog::getSaveFileName(this, tr("Save Vst List"),
                                                lastDir,
                                                tr("Vst Preset Generator file (*.vpg)"));
     if(fileName == NULL)
        return;
          
     vpgXml->WriteFile(qPrintable(fileName));
}

//-----------------------------------------------------------------------------
void MainDialog::OpenVstList()
{
     //get the name of the file
     QString fileName = QFileDialog::getOpenFileName(this, tr("Open Vst List"),
                                                lastDir,
                                                tr("Vst Preset Generator file (*.vpg)"));
     if(fileName == NULL)
        return;
     
     vpgXml->ReadFile(qPrintable(fileName));
     
     //emit an empty header
     FxHeaderStruct *invalid = new FxHeaderStruct;
     invalid->name = "";
     invalid->fxID = "";
     invalid->fxVersion = 0;
     invalid->nParams = 0;
     
     emit validHeader(invalid);
}

//-----------------------------------------------------------------------------
void MainDialog::NewVstList()
{
     //ask for saving the actual list
     int ret;
     
     if(vpgXml->dataSaved() == false)
     {
         ret = QMessageBox::question(this, tr("New Vst List..."),
                       tr("Save your actual Vst list before create a new one ?"),
                       QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
     
         if(ret == QMessageBox::Cancel)
         {
            //quit the function
            return;
         }
         else if(ret == QMessageBox::Save)
         {
            SaveVstList();
         }
     }
     
     //clear the list
     vpgXml->clearVst();
     
     //emit an empty header
     FxHeaderStruct *invalid = new FxHeaderStruct;
     invalid->name = "";
     invalid->fxID = "";
     invalid->fxVersion = 0;
     invalid->nParams = 0;
     
     emit validHeader(invalid);
     
}

//-----------------------------------------------------------------------------
void MainDialog::ShowCopyright()
{    
     //show copyright information (GNU GPL)
     
     QMessageBox mb(this);
     
     //open the licence file:
     QString text;
     
     mb.setWindowTitle(tr("About the VST Preset Generator "));
     text = tr("<h2>VST Preset Generator ");
     text += VSTPRESETGENERATORVERSION;
     text += tr("</h2>"
               "<p>Copyright &copy; 2007 Franois Mazen"
               "<p>Please visit Source Forge website for more informations:"
               "<p><a href='http://vst-preset-gen.sourceforge.net/'>http://vst-preset-gen.sourceforge.net/</a>"
               "<p>This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version."
               "<p>This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details."
               "<p>You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."
               "<p>VST is a registered trademark of Steinberg Soft- und Hardware, GmbH."
               "<p>Special thanks to: Selfik, AzralXt..."
               );
   
    
     mb.setText(text);
     
     QImage logo(VPG_LOGO);
     QPixmap pm = QPixmap::fromImage(logo);
     if (!pm.isNull())
         mb.setIconPixmap(pm);
     mb.addButton(QMessageBox::Ok);
     mb.exec();
}

//-----------------------------------------------------------------------------
void MainDialog::closeEvent(QCloseEvent* event)
{
     int ret = 0;
     if(vpgXml->dataSaved() == false)
     {
         ret = QMessageBox::question(this, tr("Close Program..."),
                       tr("Save your actual Vst list before close the application ?"),
                       QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
     
         if(ret == QMessageBox::Save)
         {
            SaveVstList();
         }
         else if(ret == QMessageBox::Cancel)
         {
            //quit the function, but not the application
            event->ignore();
            return;
         }
     }
     
     //quit the application
     event->accept();
     
}

//-----------------------------------------------------------------------------
void MainDialog::ImportParamFromVstOrPreset()
{
 	 //get the name of the file
     QString fileName = QFileDialog::getOpenFileName(this, tr("Load a Plugin VST"),
                                                lastDir,
                                                tr("VST File (*.dll *.fxp *.fxb)"));
     
     if(fileName == NULL)
        return;
        
     if(fileName.endsWith (QString("dll")))
     {
	     LoadParamFromVst(fileName);
     }
     else
     {
	  	 OpenPreset(fileName);
     }
 	 
}

//-----------------------------------------------------------------------------
void MainDialog::LoadParamFromVst(QString fileName)
{
     //get the name of the file
     //QString fileName = QFileDialog::getOpenFileName(this, tr("Load a Plugin VST"),
     //                                           lastDir,
     //                                           tr("VST File (*.dll *.fxp *.fxb)"));

     if(LoadVst(fileName) == false)
	    return;
     
     FxHeaderStruct *t_fxheaderstruct = new FxHeaderStruct;
     VstInt32 buffer_int;
     char* buffer_char = (char*) malloc(5 * sizeof(char));
     
     buffer_int = vpgVstHost->getFxId();
     import_long_char(buffer_int,buffer_char);
     
     t_fxheaderstruct->fxID = buffer_char;
     t_fxheaderstruct->fxVersion = vpgVstHost->getFxVersion();
     t_fxheaderstruct->nParams = vpgVstHost->getNumberParams();  
     t_fxheaderstruct->name = vpgVstHost->getName();
     t_fxheaderstruct->useChunk = vpgVstHost->useChunk();
     if(t_fxheaderstruct->useChunk)
     {
		   t_fxheaderstruct->fxPath = fileName;
     }
     
     //add the new Vst to our list:
     vpgXml->addVst(t_fxheaderstruct); //Maybe not ?
                   
     //send parmeters to the headers object
     emit validHeader(t_fxheaderstruct);
          
     
}


//-----------------------------------------------------------------------------
bool MainDialog::LoadVst(QString fileName)
{
     int code, ret;
     
     if(fileName == NULL)
        return false;
        
     QFileInfo info(fileName);
     lastDir = info.absolutePath();
     
     
     //replace "/" by "\\"
     //fileName.replace("/","\\\\");
     //fileName = fileName.toLower();
     //fileName = fileName.toUpper();
	 
	 	 
     //wchar_t* array = new wchar_t[fileName.size() * 4];
     //wchar_t* array = new wchar_t[1024];
     //fileName.toWCharArray(array);
     
     vpgVstHost->Close(); //Unload before loading
     //code = vpgVstHost->Load(array);
     code = vpgVstHost->Load(fileName);
     
     //delete array;
     
	 if(code != 0)
     {
  		 //error while opening
  		 vpgVstHost->Close();
         QString message;
         message = tr("Error: ");
         switch(code)
         {
 			 case ERROR_PLUGIN_MAIN:
			 	  message += tr("VST Plugin main entry not found");
			 	  break;
			 	  
	 	     case ERROR_PLUGIN_INSTANCE:
			 	  message += tr("failed to create effect instance");
			 	  break;
			 	  
	 	     case ERROR_PLUGIN_VALID:
			 	  message += tr("it's not a valid VST plugin");
			 	  break;
			 	  
 			 case ERROR_PLUGIN_LOAD:
	 	  	 default:
			 	  message += tr("failed to load VST Plugin library");
			 	  message += tr("\n");
			 	  message += fileName;
			 	  break;
         }
  		     
         
         ret = QMessageBox::critical(this, tr("Loading plugin VST"), message);                        
         return false; //exit
     }
     
     return true;
}

