/*
-------------------------------------------------------------------------------
    This file is a part of the Vst Preset Generator
    Copyright (C) 2007  Francois Mazen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-------------------------------------------------------------------------------
  Name: vpgvsthost.cpp
  Author: Francois Mazen
  Date: 13/06/07 17:50
  Description: The class VpgVstHost can open a plug-in Vst in order to load
               informations about the VST and avoid the "opaque chunk method"
               in the presets files
-------------------------------------------------------------------------------
*/

#include <QtGui>

#include "vpgvsthost.h"

#include "vst2.x/aeffectx.h"





//-----------------------------------------------------------------------------
//VpgVstHost class
//-----------------------------------------------------------------------------

VpgVstHost::VpgVstHost() 
{
    ptrPlug = NULL;
    //libhandle = NULL;
    library = new QLibrary();
}

//-----------------------------------------------------------------------------
/*int VpgVstHost::Load(const wchar_t* filename)
{
    	
    
	libhandle = LoadLibrary(filename);

	
	if (libhandle == NULL)
	{
		return ERROR_PLUGIN_LOAD;
	}
	
	//DLL was loaded OK
	AEffect* (__cdecl* getNewPlugInstance)(audioMasterCallback);
	getNewPlugInstance = (AEffect*(__cdecl*)(audioMasterCallback))GetProcAddress(libhandle, "main");
	
	if (getNewPlugInstance == NULL)
	{
		return ERROR_PLUGIN_MAIN;
	}
	
	//main function located OK
	ptrPlug = getNewPlugInstance(host);
	
	if (ptrPlug == NULL)
	{
		return ERROR_PLUGIN_INSTANCE;
	}
	
	if (ptrPlug->magic != kEffectMagic)
	{
	   return ERROR_PLUGIN_VALID;
	}
	
	
	
	
	return 0;
}    */

int VpgVstHost::Load(QString filename)
{
 	library->setFileName(filename);
 	
 	if(!library->load())
 	{
		return ERROR_PLUGIN_LOAD;
    }
    
    //DLL was loaded OK
	AEffect* (__cdecl* getNewPlugInstance)(audioMasterCallback);
	getNewPlugInstance = (AEffect*(__cdecl*)(audioMasterCallback)) library->resolve("main");
    
    if (getNewPlugInstance == NULL)
	{
		return ERROR_PLUGIN_MAIN;
	}
	
	//main function located OK
	ptrPlug = getNewPlugInstance(host);
	
	if (ptrPlug == NULL)
	{
		return ERROR_PLUGIN_INSTANCE;
	}
	
	if (ptrPlug->magic != kEffectMagic)
	{
	   return ERROR_PLUGIN_VALID;
	}
	
	
	
	
	return 0;
}

//-----------------------------------------------------------------------------
void VpgVstHost::Close()
{
 	 if(ptrPlug)
 	 {
			ptrPlug->dispatcher(ptrPlug, effClose, 0, 0, 0, 0);
			ptrPlug = NULL;
     }
	 
	 
     if(library->isLoaded())
     {
	        library->unload();
     }
     
}

//-----------------------------------------------------------------------------
bool VpgVstHost::isVstLoaded()
{
 	 return ( (library->isLoaded()) && (ptrPlug != NULL));
}

//-----------------------------------------------------------------------------
VstInt32 VpgVstHost::getFxId()
{
 	  if(!ptrPlug)
		  return 0;
	     
      return (ptrPlug->uniqueID);
}

//-----------------------------------------------------------------------------
VstInt32 VpgVstHost::getFxVersion()
{
 	  if(!ptrPlug)
		  return 0;
	     
      return (ptrPlug->version);
}		 

//-----------------------------------------------------------------------------
VstInt32 VpgVstHost::getNumberParams()
{
      if(!ptrPlug)
		  return 0;
	     
      return (ptrPlug->numParams);
}	

//-----------------------------------------------------------------------------
VstInt32 VpgVstHost::getNumberPrograms()
{
      if(!ptrPlug)
		  return 0;
	     
      return (ptrPlug->numPrograms);
}		  		 

//-----------------------------------------------------------------------------
char* VpgVstHost::getName()
{
      if(!ptrPlug)
		  return NULL;
      //dispatcher(ptrPlug,effGetVstVersion,0,0,NULL,0.0f)
      char* buffer = new char[kVstMaxEffectNameLen];
      ptrPlug->dispatcher(ptrPlug,effGetEffectName,0,0,buffer,0.0f);
      return (buffer);
}	

//-----------------------------------------------------------------------------
void VpgVstHost::getParamName(VstInt32 index, char* buffer)
{
 	  //effGetParamName,	
	  ///< [ptr]: char buffer for parameter name, limited to #kVstMaxParamStrLen  
	  //@see AudioEffect::getParameterName
	  
	  //char* buffer = new char[kVstMaxParamStrLen];
	  
	  ptrPlug->dispatcher(ptrPlug,effGetParamName,index,0,buffer,0.0f);
	  
}


//-----------------------------------------------------------------------------
bool VpgVstHost::useChunk()
{
 	 //effFlagsProgramChunks
 	 if ( (ptrPlug->flags & effFlagsProgramChunks) != 0)
 	    return true;
 	    
     return false;
}

//-----------------------------------------------------------------------------
void VpgVstHost::setParameter(long index, float value)
{
 	 if (ptrPlug == NULL)
 	 	return;
 	 	
 	 ptrPlug->setParameter(ptrPlug, index, value);
}

//-----------------------------------------------------------------------------
VstInt32 VpgVstHost::getProgramChunk(void** chunk)
{		
       ///effGetChunk:
 	   ///  [ptr]: void** for chunk data address 
       ///  [index]: 0 for bank, 1 for program  @see AudioEffect::getChunk
 	   	
	   //(AEffect* effect, VstInt32 opcode, VstInt32 index, VstIntPtr value, void* ptr, float opt)
	   long ret = ptrPlug->dispatcher(ptrPlug,effGetChunk,1,0,chunk,0.0f);	
	   
	   return ret;
}

//-----------------------------------------------------------------------------
VstInt32 VpgVstHost::getBankChunk(void** chunk)
{		
       ///effGetChunk:
 	   ///  [ptr]: void** for chunk data address 
       ///  [index]: 0 for bank, 1 for program  @see AudioEffect::getChunk
 	   	
	   //(AEffect* effect, VstInt32 opcode, VstInt32 index, VstIntPtr value, void* ptr, float opt)
	   long ret = ptrPlug->dispatcher(ptrPlug,effGetChunk,0,0,chunk,0.0f);	
	   
	   return ret;
}

//-----------------------------------------------------------------------------
void VpgVstHost::setProgramName(const char* name, long index)
{
 	 ptrPlug->dispatcher(ptrPlug,effSetProgramName,index,0,(void*)name,0.0f);
}

//-----------------------------------------------------------------------------
void VpgVstHost::setProgram(long num)
{
 	 ptrPlug->dispatcher(ptrPlug,effSetProgram,0,num,0,0.0f);
}

//-----------------------------------------------------------------------------
//host callback function
//this is called directly by the plug-in!!
//-----------------------------------------------------------------------------

VstIntPtr VSTCALLBACK host(AEffect *effect, VstInt32 opcode, VstInt32 index, VstInt32 value, void *ptr, float opt)
{
	VstIntPtr retval = 0;

	switch (opcode)
	{
		//VST 1.0 opcodes
		case audioMasterVersion:
			//Input values:
			//none

			//Return Value:
			//0 or 1 for old version
			//2 or higher for VST2.0 host?
			//cout << "plug called audioMasterVersion" << endl;
			retval=2;
			break;

		case audioMasterAutomate:
			//Input values:
			//<index> parameter that has changed
			//<opt> new value

			//Return value:
			//not tested, always return 0

			//NB - this is called when the plug calls
			//setParameterAutomated

			//cout << "plug called audioMasterAutomate" << endl;
			break;

		case audioMasterCurrentId:
			//Input values:
			//none

			//Return Value
			//the unique id of a plug that's currently loading
			//zero is a default value and can be safely returned if not known
			//cout << "plug called audioMasterCurrentId" << endl;
			break;

		case audioMasterIdle:
			//Input values:
			//none

			//Return Value
			//not tested, always return 0

			//NB - idle routine should also call effEditIdle for all open editors
			//Sleep(1);
			//cout << "plug called audioMasterIdle" << endl;
			break;

		case audioMasterPinConnected:
			//Input values:
			//<index> pin to be checked
			//<value> 0=input pin, non-zero value=output pin

			//Return values:
			//0=true, non-zero=false
			//cout << "plug called audioMasterPinConnected" << endl;
			break;

		//VST 2.0 opcodes
		/*case audioMasterWantMidi:
			//Input Values:
			//<value> filter flags (which is currently ignored, no defined flags?)

			//Return Value:
			//not tested, always return 0
			cout << "plug called audioMasterWantMidi" << endl;
			break;*/

		case audioMasterGetTime:
			//Input Values:
			//<value> should contain a mask indicating which fields are required
			//...from the following list?
			//kVstNanosValid
			//kVstPpqPosValid
			//kVstTempoValid
			//kVstBarsValid
			//kVstCyclePosValid
			//kVstTimeSigValid
			//kVstSmpteValid
			//kVstClockValid

			//Return Value:
			//ptr to populated const VstTimeInfo structure (or 0 if not supported)

			//NB - this structure will have to be held in memory for long enough
			//for the plug to safely make use of it
			//cout << "plug called audioMasterGetTime" << endl;
			break;

		case audioMasterProcessEvents:
			//Input Values:
			//<ptr> Pointer to a populated VstEvents structure

			//Return value:
			//0 if error
			//1 if OK
			//cout << "plug called audioMasterProcessEvents" << endl;
			break;

		/*case audioMasterSetTime:
			//IGNORE!
			break;*/

		/*case audioMasterTempoAt:
			//Input Values:
			//<value> sample frame location to be checked

			//Return Value:
			//tempo (in bpm * 10000)
			cout << "plug called audioMasterTempoAt" << endl;
			break;*/

		/*case audioMasterGetNumAutomatableParameters:
			//Input Values:
			//None

			//Return Value:
			//number of automatable parameters
			//zero is a default value and can be safely returned if not known

			//NB - what exactly does this mean? can the host set a limit to the
			//number of parameters that can be automated?
			cout << "plug called audioMasterGetNumAutomatableParameters" << endl;
			break;*/

		/*case audioMasterGetParameterQuantization:
			//Input Values:
			//None

			//Return Value:
			//integer value for +1.0 representation,
			//or 1 if full single float precision is maintained
			//in automation.

			//NB - ***possibly bugged***
			//Steinberg notes say "parameter index in <value> (-1: all, any)"
			//but in aeffectx.cpp no parameters are taken or passed
			cout << "plug called audioMasterGetParameterQuantization" << endl;
			break;*/

		case audioMasterIOChanged:
			//Input Values:
			//None

			//Return Value:
			//0 if error
			//non-zero value if OK
			//cout << "plug called audioMasterIOChanged" << endl;
			break;

		/*case audioMasterNeedIdle:
			//Input Values:
			//None

			//Return Value:
			//0 if error
			//non-zero value if OK

			//NB plug needs idle calls (outside its editor window)
			//this means that effIdle must be dispatched to the plug
			//during host idle process and not effEditIdle calls only when its
			//editor is open
			//Check despatcher notes for any return codes from effIdle
			cout << "plug called audioMasterNeedIdle" << endl;
			break;*/

		case audioMasterSizeWindow:
			//Input Values:
			//<index> width
			//<value> height

			//Return Value:
			//0 if error
			//non-zero value if OK
			//cout << "plug called audioMasterSizeWindow" << endl;
			break;

		case audioMasterGetSampleRate:
			//Input Values:
			//None

			//Return Value:
			//not tested, always return 0

			//NB - Host must despatch effSetSampleRate to the plug in response
			//to this call
			//Check despatcher notes for any return codes from effSetSampleRate
			//cout << "plug called audioMasterGetSampleRate" << endl;
			effect->dispatcher(effect,effSetSampleRate,0,0,NULL,samplerate);
			break;

		case audioMasterGetBlockSize:
			//Input Values:
			//None

			//Return Value:
			//not tested, always return 0

			//NB - Host must despatch effSetBlockSize to the plug in response
			//to this call
			//Check despatcher notes for any return codes from effSetBlockSize
			//cout << "plug called audioMasterGetBlockSize" << endl;
			effect->dispatcher(effect,effSetBlockSize,0,blocksize,NULL,0.0f);

			break;

		case audioMasterGetInputLatency:
			//Input Values:
			//None

			//Return Value:
			//input latency (in sampleframes?)
			//cout << "plug called audioMasterGetInputLatency" << endl;
			break;

		case audioMasterGetOutputLatency:
			//Input Values:
			//None

			//Return Value:
			//output latency (in sampleframes?)
			//cout << "plug called audioMasterGetOutputLatency" << endl;
			break;

		/*case audioMasterGetPreviousPlug:
			//Input Values:
			//None

			//Return Value:
			//pointer to AEffect structure or NULL if not known?

			//NB - ***possibly bugged***
			//Steinberg notes say "input pin in <value> (-1: first to come)"
			//but in aeffectx.cpp no parameters are taken or passed
			cout << "plug called audioMasterGetPreviousPlug" << endl;
			break;*/

		/*case audioMasterGetNextPlug:
			//Input Values:
			//None

			//Return Value:
			//pointer to AEffect structure or NULL if not known?

			//NB - ***possibly bugged***
			//Steinberg notes say "output pin in <value> (-1: first to come)"
			//but in aeffectx.cpp no parameters are taken or passed
			cout << "plug called audioMasterGetNextPlug" << endl;
			break;*/

		/*case audioMasterWillReplaceOrAccumulate:
			//Input Values:
			//None

			//Return Value:
			//0: not supported
			//1: replace
			//2: accumulate
			cout << "plug called audioMasterWillReplaceOrAccumulate" << endl;
			break;*/

		case audioMasterGetCurrentProcessLevel:
			//Input Values:
			//None

			//Return Value:
			//0: not supported,
			//1: currently in user thread (gui)
			//2: currently in audio thread (where process is called)
			//3: currently in 'sequencer' thread (midi, timer etc)
			//4: currently offline processing and thus in user thread
			//other: not defined, but probably pre-empting user thread.
			//cout << "plug called audioMasterGetCurrentProcessLevel" << endl;
			break;

		case audioMasterGetAutomationState:
			//Input Values:
			//None

			//Return Value:
			//0: not supported
			//1: off
			//2:read
			//3:write
			//4:read/write
			//cout << "plug called audioMasterGetAutomationState" << endl;
			break;

		case audioMasterGetVendorString:
			//Input Values:
			//<ptr> string (max 64 chars) to be populated

			//Return Value:
			//0 if error
			//non-zero value if OK
			//cout << "plug called audioMasterGetVendorString" << endl;
			break;

		case audioMasterGetProductString:
			//Input Values:
			//<ptr> string (max 64 chars) to be populated

			//Return Value:
			//0 if error
			//non-zero value if OK
			//cout << "plug called audioMasterGetProductString" << endl;
			break;

		case audioMasterGetVendorVersion:
			//Input Values:
			//None

			//Return Value:
			//Vendor specific host version as integer
			//cout << "plug called audioMasterGetVendorVersion" << endl;
			break;

		case audioMasterVendorSpecific:
			//Input Values:
			//<index> lArg1
			//<value> lArg2
			//<ptr> ptrArg
			//<opt>	floatArg

			//Return Values:
			//Vendor specific response as integer
			//cout << "plug called audioMasterVendorSpecific" << endl;
			break;

		/*case audioMasterSetIcon:
			//IGNORE
			break;*/

		case audioMasterCanDo:
			//Input Values:
			//<ptr> predefined "canDo" string

			//Return Value:
			//0 = Not Supported
			//non-zero value if host supports that feature

			//NB - Possible Can Do strings are:
			//"sendVstEvents",
			//"sendVstMidiEvent",
			//"sendVstTimeInfo",
			//"receiveVstEvents",
			//"receiveVstMidiEvent",
			//"receiveVstTimeInfo",
			//"reportConnectionChanges",
			//"acceptIOChanges",
			//"sizeWindow",
			//"asyncProcessing",
			//"offline",
			//"supplyIdle",
			//"supportShell"
			//cout << "plug called audioMasterCanDo" << endl;

			if (strcmp((char*)ptr,"sendVstEvents")==0 ||
				strcmp((char*)ptr,"sendVstMidiEvent")==0 ||
				strcmp((char*)ptr,"supplyIdle")==0)
			{
				retval=1;
			}
			else
			{
				retval=0;
			}

			break;

		case audioMasterGetLanguage:
			//Input Values:
			//None

			//Return Value:
			//kVstLangEnglish
			//kVstLangGerman
			//kVstLangFrench
			//kVstLangItalian
			//kVstLangSpanish
			//kVstLangJapanese
			//cout << "plug called audioMasterGetLanguage" << endl;
			retval=1;
			break;
/*
		MAC SPECIFIC?

		case audioMasterOpenWindow:
			//Input Values:
			//<ptr> pointer to a VstWindow structure

			//Return Value:
			//0 if error
			//else platform specific ptr
			cout << "plug called audioMasterOpenWindow" << endl;
			break;

		case audioMasterCloseWindow:
			//Input Values:
			//<ptr> pointer to a VstWindow structure

			//Return Value:
			//0 if error
			//Non-zero value if OK
			cout << "plug called audioMasterCloseWindow" << endl;
			break;
*/
		case audioMasterGetDirectory:
			//Input Values:
			//None

			//Return Value:
			//0 if error
			//FSSpec on MAC, else char* as integer

			//NB Refers to which directory, exactly?
			//cout << "plug called audioMasterGetDirectory" << endl;
			break;

		case audioMasterUpdateDisplay:
			//Input Values:
			//None

			//Return Value:
			//Unknown
			//cout << "plug called audioMasterUpdateDisplay" << endl;
			break;
	}

	return retval;
};





